/*
 * dbmuser is just add-on piece of code, which can be used for 
 * ndbm group file management.
 *
 * Copyright (c) by Juha Ylitalo (juha.o.ylitalo@ntc.nokia.com)
 */
#include <fcntl.h>
#include <ndbm.h>
#include <stdio.h>
#include <strings.h>
#include <errno.h>

int main( int argc , char** argv ) {

  if ( argc < 3 ) {
    fprintf( stderr , "Usage: dbmuser <operation> <database> <user> <group1> <group2>\n");
    fprintf( stderr , "operations: add | show | delete | size\n" );
  } else if ( argc == 3 ) {
    DBM* f;
    char* operation = argv[ 1 ];
    char* dbm_file = argv[ 2 ];

    if ( strcmp( operation , "size" ) == 0 ) {
      int i;
      datum key;
      
      f = dbm_open( dbm_file , O_RDONLY , 0664 );
      for ( i = 0 , key = dbm_firstkey( f );  
	    key.dptr !=  NULL ;  i++ , key  = dbm_nextkey( f ) )
	;
      fprintf( stderr, "%s has %d items.\n" , dbm_file , i );
    }
    if ( f != NULL ) {
      dbm_close( f );
    }
  } else {
    DBM* f = NULL;
    char* operation = argv[ 1 ];
    char* dbm_file = argv[ 2 ];
    char* user = argv[ 3 ];

    if ( strcmp( operation , "add" ) == 0 ) {
      datum key;
      datum dat;
      char buf[1024];
      int i;

      f = dbm_open( dbm_file , O_RDWR | O_CREAT, 0664 );
      strcpy( buf , " " );
      for ( i = 4 ; i < argc ; i++ ) {
	sprintf( buf , "%s%s ", buf , argv[ i ] );
      } /* for */
      key.dptr = strdup( user );
      key.dsize = ( strlen( user ) + 1 ) * sizeof( char );
      dat.dptr = strdup( buf );
      dat.dsize = ( strlen( buf ) + 1) * sizeof( char );

      dbm_store( f , key , dat , DBM_REPLACE );

      printf( "User (%s), who belongs to groups: %s, was added to %s.\n",
	      user , buf , dbm_file );
      if ( errno != 0 ) {
	perror( "dbmuser :: " );
	exit( 1 );
      } /* if */
    } else if ( strcmp( operation , "show" ) == 0 ) {
      char buf[ 1024 ];
      int len;
      datum key = { user , ( strlen( user ) + 1 ) * sizeof( char )  };
      datum dat;
      
      f = dbm_open( dbm_file , O_RDONLY , 0664 );
      dat = dbm_fetch( f , key );

      len = dat.dsize / sizeof( char );
      strncpy( buf , dat.dptr , len );
      buf[ len ] = '\0';
      printf( "User belongs to following groups: %s\n" , buf );
    } else if ( strcmp( operation , "delete" ) == 0 ) {
      datum key = { user , ( strlen( user ) + 1 ) * sizeof( char ) };
      
      f = dbm_open( dbm_file , O_RDWR , 0664 );
      dbm_delete( f , key );
      printf( "User (%s) deleted from %s\n" , user , dbm_file );
    } /* if */
    if ( f != NULL ) {
      dbm_close( f );
    }
  } /* if */
} /* main */
